<?xml version="1.0"?>
<xsl:stylesheet 
  version='1.0' 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:exslt="http://exslt.org/common"
  extension-element-prefixes="exslt">

  <!-- Convert sections of an EtreCheck report into HTML 
       representations. -->
  <xsl:output method="text" encoding="UTF-8"/> 
 
  <!-- French isn't done yet. -->
  <xsl:param name="language">en</xsl:param>
  
  <!-- Language-specific data. -->
  <xsl:variable 
    name="strings" 
    select="document(concat('html_', $language, '.xml'))/strings"/>
  
  <!-- Language-specific issues. -->
  <xsl:variable 
    name="issues" 
    select="document(concat('Issues_', $language, '.xml'))/issues"/>

  <!-- Grab some convenient lookup tables. -->
  <xsl:variable name="performance" select="$strings/performance_rating"/>

  <xsl:variable name="batteryhealth" select="$strings/batteryhealth"/>

  <xsl:variable name="statusattributes" select="$strings/status"/>
  
  <xsl:variable name="pad">
    <xsl:text>                                                   </xsl:text>
  </xsl:variable>
  
  <xsl:variable name= "launchdstatuswidth">
    <xsl:call-template name="maxwidth">
      <xsl:with-param name="nodes" select="$strings/launchd/status/value"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name= "kernelextensionstatuswidth">
    <xsl:call-template name="maxwidth">
      <xsl:with-param 
        name="nodes" select="$strings/kernelextensions/status/value"/>
    </xsl:call-template>
  </xsl:variable>

  <!-- Newlines are special in XSL. -->
  <xsl:variable name="newline">
    <xsl:text>&#xa;</xsl:text>
  </xsl:variable>

  <!-- Is is only for the full report. -->
  <xsl:template match='/etrecheck'>
  
    <xsl:apply-templates select="header"/>
    <xsl:call-template name="major"/>
    <xsl:call-template name="minor"/>
    <xsl:apply-templates select="hardware"/>
    <xsl:apply-templates select="video"/>
    <xsl:apply-templates select="disk"/>
    <xsl:apply-templates select="virtualvolume"/>
    <xsl:apply-templates select="network"/>
    <xsl:apply-templates select="systemsoftware"/>
    <xsl:apply-templates select="configurationfiles"/>
    <xsl:apply-templates select="security"/>
    <xsl:apply-templates select="adware"/>
    <xsl:apply-templates select="unsigned"/>
    <xsl:apply-templates select="kernelextensions"/>
    <xsl:apply-templates select="startupitems"/>
    <xsl:apply-templates select="systemlaunchagents"/>
    <xsl:apply-templates select="systemlaunchdaemons"/>
    <xsl:apply-templates select="launchagents"/>
    <xsl:apply-templates select="launchdaemons"/>
    <xsl:apply-templates select="userlaunchagents"/>
    <xsl:apply-templates select="loginitems"/>
    <xsl:apply-templates select="internetplugins"/>
    <xsl:apply-templates select="userinternetplugins"/>
    <xsl:apply-templates select="audioplugins"/>
    <xsl:apply-templates select="useraudioplugins"/>
    <xsl:apply-templates select="safariextensions"/>
    <xsl:apply-templates select="preferencepanes"/>
    <xsl:apply-templates select="timemachine"/>
    <xsl:apply-templates select="cpu"/>
    <xsl:apply-templates select="memory"/>
    <xsl:apply-templates select="networkusage"/>
    <xsl:apply-templates select="energy"/>
    <xsl:apply-templates select="vm"/>
    <xsl:apply-templates select="install"/>
    <xsl:apply-templates select="cleanup"/>
    <xsl:apply-templates select="diagnostics"/>
    <xsl:apply-templates select="etrecheckdeletedfiles"/>
    
    <!-- Add some plain text at the end to keep the ASC edit window
         from going into a funky table-edit mode. -->
    <xsl:value-of select="$newline"/>
    <xsl:value-of select="$strings/endofreport"/>
    <xsl:value-of select="$newline"/>

  </xsl:template>

  <!-- EtreCheck stats. -->
  <xsl:template match="header">
  
    <xsl:variable name="performance" select="performance"/>
    
    <xsl:apply-templates select="$strings/header/version" mode="label"/>
    <xsl:value-of select="version"/>
    <xsl:text> (</xsl:text>
    <xsl:value-of select="build"/>
    <xsl:text>)</xsl:text>
    <xsl:value-of select="$newline"/>
    
    <xsl:apply-templates 
      select="$strings/header/report_generated" mode="label"/>

    <xsl:value-of select="date"/>
    <xsl:value-of select="$newline"/>
    
    <xsl:apply-templates 
      select="$strings/header/download_etrecheck" mode="label"/>
    
    <xsl:value-of select="$newline"/>

    <xsl:apply-templates select="$strings/header/runtime" mode="label"/>
    <xsl:value-of select="runtime"/>
    <xsl:value-of select="$newline"/>

    <xsl:apply-templates 
      select="$strings/header/performance" mode="label"/>

    <xsl:value-of 
      select="$strings/header/performance_rating/value[@key = $performance]"/>

    <xsl:value-of select="$newline"/>

    <xsl:value-of select="$newline"/>
    
    <xsl:apply-templates select="$strings/header/problem" mode="label"/>
    <xsl:value-of select="problem"/>
    <xsl:value-of select="$newline"/>

    <!-- Description is optional. -->
    <xsl:if test="problemdescription">
    
      <xsl:apply-templates 
        select="$strings/header/problemdescription" mode="label"/>
      
      <xsl:value-of select="$newline"/>
      
      <!-- ASC scrambles multi-line elements. Do the scrambling myself so
           I have some control over the outcome. -->
      <xsl:call-template name="split">
        <xsl:with-param name="text">
          <xsl:value-of select="problemdescription"/>
        </xsl:with-param>
      </xsl:call-template>
      
    </xsl:if>
    
    <xsl:value-of select="$newline"/>
    
  </xsl:template>

  <xsl:include href="Issues.xsl" />

  <!-- Major issues. -->
  <xsl:template name="major">
  
    <xsl:variable name="content">
      <xsl:call-template name="majorissues"/>
    </xsl:variable>

    <xsl:choose>
    
      <xsl:when test="string-length($content) = 0">
        <xsl:call-template name="custom_header">
          <xsl:with-param name="text">
            <xsl:value-of select="$issues/major/none/node()"/>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      
      <xsl:otherwise>
      
        <xsl:call-template name="header">
          <xsl:with-param name="key">major</xsl:with-param>
        </xsl:call-template>
        
        <xsl:call-template name="indent">
          <xsl:with-param name="indent">2</xsl:with-param>
        </xsl:call-template>
        
        <xsl:copy-of select="$issues/major/problemsfound/node()"/>
        <xsl:value-of select="$newline"/>

        <xsl:copy-of select="$content"/>
          
      </xsl:otherwise>
    
    </xsl:choose>
  
    <xsl:value-of select="$newline"/>
    
  </xsl:template>
    
  <!-- Minor issues. -->
  <xsl:template name="minor">
  
    <xsl:variable name="content">
      <xsl:call-template name="minorissues"/>
    </xsl:variable>

    <xsl:choose>
    
      <xsl:when test="string-length($content) = 0">
        <xsl:call-template name="custom_header">
          <xsl:with-param name="text">
            <xsl:value-of select="$issues/minor/none/node()"/>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      
      <xsl:otherwise>
      
        <xsl:call-template name="header">
          <xsl:with-param name="key">minor</xsl:with-param>
        </xsl:call-template>
        
        <xsl:call-template name="indent">
          <xsl:with-param name="indent">2</xsl:with-param>
        </xsl:call-template>

        <xsl:copy-of select="$issues/minor/problemsfound/node()"/>
        <xsl:value-of select="$newline"/>

        <xsl:copy-of select="$content"/>
          
      </xsl:otherwise>
    
    </xsl:choose>

    <xsl:value-of select="$newline"/>
  
  </xsl:template>
  
  <!-- Emit an issue. -->
  <xsl:template name="issue">
    <xsl:param name="type"/>
    <xsl:param name="name"/>
    
    <xsl:variable 
      name="issuenodes" select="$issues/*[local-name() = $type]"/>
    
    <xsl:variable 
      name="issue" select="$issuenodes/*[local-name() = $name]"/>
    
    <xsl:call-template name="indent">
      <xsl:with-param name="indent">2</xsl:with-param>
    </xsl:call-template>
    <xsl:value-of select="$issue/title"/>
    <xsl:text> - </xsl:text>
    <xsl:value-of select="$issue/description"/>
    <xsl:value-of select="$newline"/>

  </xsl:template>

  <!-- Hardware information. -->
  <xsl:template match="hardware">
  
    <xsl:variable name="healthvalue" select="batteryhealth"/>

    <xsl:call-template name="header"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent">2</xsl:with-param>
    </xsl:call-template>

    <xsl:value-of select="marketingname"/>

    <xsl:choose>
      <xsl:when test="vintage = 'vintage'">
        <xsl:text> - </xsl:text>
        <xsl:copy-of select="$strings/hardware/vintage/node()"/>
      </xsl:when>
      
      <xsl:when test="vintage = 'obsolete'">
        <xsl:text> - </xsl:text>
        <xsl:copy-of select="$strings/hardware/obsolete/node()"/>
      </xsl:when>

      <xsl:otherwise/>
      
    </xsl:choose>

    <xsl:value-of select="$newline"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent">2</xsl:with-param>
    </xsl:call-template>

    <xsl:value-of select="name"/>
    <xsl:text> </xsl:text>
    <xsl:value-of select="$strings/hardware/model"/>
    <xsl:value-of select="model"/>
    <xsl:value-of select="$newline"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent">2</xsl:with-param>
    </xsl:call-template>

    <xsl:value-of select="cpucount"/>
    <xsl:text> </xsl:text>
    <xsl:apply-templates select="speed" mode="units"/>
    <xsl:text> </xsl:text>
    
    <xsl:call-template name="format">
      <xsl:with-param name="format" 
        select="$strings/hardware/cpu_format"/>
      <xsl:with-param name="value" select="cpu_type|cpucode|corecount"/>
    </xsl:call-template>
    <xsl:value-of select="$newline"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent">2</xsl:with-param>
    </xsl:call-template>

    <xsl:apply-templates select="ram"/>
    <xsl:text> RAM</xsl:text>
       
    <xsl:text> </xsl:text>
      
    <xsl:choose> 
    
      <xsl:when test="upgradeable = 'true'">     
        <xsl:value-of select="$strings/hardware/upgradeable"/>
      </xsl:when>
      
      <xsl:otherwise>
        <xsl:value-of select="$strings/hardware/not_upgradeable"/>
      </xsl:otherwise>
      
    </xsl:choose>
      
    <xsl:value-of select="$newline"/>

    <xsl:for-each select="memorybanks/memorybank">
      <xsl:call-template name="indent">
        <xsl:with-param name="indent">4</xsl:with-param>
      </xsl:call-template>

      <xsl:value-of select="identifier"/>
      <xsl:value-of select="$newline"/>

      <xsl:call-template name="indent">
        <xsl:with-param name="indent">6</xsl:with-param>
      </xsl:call-template>

      <xsl:apply-templates select="size" mode="units"/>
      <xsl:text> </xsl:text>
      <xsl:value-of select="type"/>
      <xsl:text> </xsl:text>
      <xsl:apply-templates select="speed" mode="units"/>
      <xsl:text> </xsl:text>
      <xsl:value-of select="status"/>
      <xsl:value-of select="$newline"/>
    </xsl:for-each>
    
    <xsl:if test="batteryhealth and batterycyclecount and batterypercent">
      <xsl:call-template name="indent">
        <xsl:with-param name="indent">2</xsl:with-param>
      </xsl:call-template>

      <xsl:value-of select="$strings/hardware/battery"/>
      <xsl:value-of select="$strings/hardware/health"/>
      <xsl:text> = </xsl:text>
      <xsl:value-of 
        select="$strings/hardware/batteryhealth/value[@key = $healthvalue]"/>
      <xsl:text> - </xsl:text>
      <xsl:value-of select="$strings/hardware/cycle_count"/>
      <xsl:text> = </xsl:text>
      <xsl:value-of select="batterycyclecount"/>
      <xsl:value-of select="$newline"/>
      <xsl:value-of select="$newline"/>
    </xsl:if>
    
  </xsl:template>
 
  <!-- Video information. -->
  <xsl:template match="video">
  
    <xsl:call-template name="header"/>

    <xsl:for-each select="gpu">
        
      <xsl:call-template name="indent">
        <xsl:with-param name="indent">2</xsl:with-param>
      </xsl:call-template>
      
      <xsl:value-of select="name"/>
      <xsl:if test="vram">
        <xsl:text> - VRAM: </xsl:text>
        <xsl:apply-templates select="vram" mode="units"/>
      </xsl:if>
      <xsl:value-of select="$newline"/>
    
      <!-- Report displays, if any. -->
      <xsl:if test="count(displays/display) &gt; 0">
        <xsl:for-each select="displays/display">
          <xsl:call-template name="indent">
            <xsl:with-param name="indent">4</xsl:with-param>
          </xsl:call-template>

          <xsl:value-of select="name"/>
          <xsl:text> </xsl:text>
          <xsl:value-of select="resolution"/>
          <xsl:value-of select="$newline"/>
        </xsl:for-each>
      </xsl:if>
    </xsl:for-each>
      
    <xsl:value-of select="$newline"/>

  </xsl:template>
  
  <!-- Drive information. -->
  <xsl:template match="disk">
  
    <xsl:call-template name="header"/>

    <xsl:apply-templates select="drive"/>

    <xsl:value-of select="$newline"/>
    
  </xsl:template>

  <!-- Physical drive. -->
  <xsl:template match="drive">
  
    <xsl:variable name="type" select="type"/>
    <xsl:variable name="mechanism" select="mechanism"/>
    
    <xsl:if test="position() &gt; 1">
      <xsl:value-of select="$newline"/>
    </xsl:if>
    
    <xsl:call-template name="indent">
      <xsl:with-param name="indent">2</xsl:with-param>
    </xsl:call-template>

    <xsl:value-of select="device"/>
    <xsl:text> - </xsl:text>
    <xsl:value-of select="model"/>
    <xsl:text> </xsl:text>
    <xsl:apply-templates select="size" mode="units"/>
    <xsl:text> (</xsl:text>
    <xsl:value-of select="$strings/drive/mechanisms/value[@key = $mechanism]"/>
    
    <xsl:if test="TRIM">
      <xsl:value-of select="$strings/drive/trim"/>
      <xsl:apply-templates select="TRIM"/>
    </xsl:if>
    
    <xsl:text>) </xsl:text>
    <xsl:value-of select="$newline"/>
    
    <xsl:call-template name="indent">
      <xsl:with-param name="indent">2</xsl:with-param>
    </xsl:call-template>

    <xsl:choose>
      <xsl:when test="internal = 'true'">
        <xsl:value-of select="$strings/drive/internal"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$strings/drive/external"/>
      </xsl:otherwise>
    </xsl:choose>
    
    <xsl:text> </xsl:text>
    <xsl:value-of select="bus"/>
      
    <xsl:if test="busspeed">
      <xsl:text> </xsl:text>
      <xsl:value-of select="busspeed"/>
    </xsl:if>

    <xsl:if test="type">
      <xsl:text> </xsl:text>
      <xsl:value-of select="$strings/drive/types/value[@key = $type]"/>
    </xsl:if>
      
    <xsl:value-of select="$newline"/>
    
    <xsl:choose>
      <xsl:when test="smartstatus = 'Verified'"/>
      <xsl:when test="smartstatus = 'Not Supported'"/>
      <xsl:otherwise>
        <xsl:call-template name="indent">
          <xsl:with-param name="indent">2</xsl:with-param>
        </xsl:call-template>
        <xsl:value-of select="$strings/drive/smart_status"/>
        <xsl:value-of select="smartstatus"/>
        <xsl:value-of select="$newline"/>
      </xsl:otherwise>
    </xsl:choose>
    
    <xsl:if test="volumes/device">
      <xsl:call-template name="volumes">
        <xsl:with-param name="volumes" select="volumes"/>
        <xsl:with-param name="indent">4</xsl:with-param>
      </xsl:call-template>
    </xsl:if>

  </xsl:template>

  <!-- Other volumes information. -->
  <xsl:template match="virtualvolume">
  
    <xsl:variable name="othervolumes">
      <xsl:for-each select="volume">
        <xsl:variable name="device" select="device"/>
        <xsl:choose>
          <xsl:when test="mountpoint">
            <xsl:copy-of select="."/>
          </xsl:when>
          <xsl:otherwise/>
        </xsl:choose>
      </xsl:for-each>
    </xsl:variable>
    
    <xsl:if test="count($othervolumes) &gt; 0">
    
      <xsl:call-template name="header"/>

      <xsl:apply-templates select="exslt:node-set($othervolumes)"/>
      
    </xsl:if>
    
    <xsl:value-of select="$newline"/>

  </xsl:template>

  <!-- A mini-list of devices. -->
  <xsl:template name="volumes">
    <xsl:param name="indent">0</xsl:param>
    <xsl:param name="volumes"/>
  
    <xsl:for-each select="$volumes/device">
      <xsl:call-template name="device">
        <xsl:with-param name="indent" select="$indent"/>
        <xsl:with-param name="device" select="."/>
      </xsl:call-template>
    </xsl:for-each>
    
  </xsl:template>
  
  <!-- A mini-list of devices. -->
  <xsl:template name="device">
    <xsl:param name="indent">10</xsl:param>
    <xsl:param name="device"/>
    
    <xsl:variable 
      name="volume" 
      select="/etrecheck/virtualvolume/volume[device = $device]"/>
    
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="$indent"/>
    </xsl:call-template>

    <xsl:value-of select="$device"/>

    <xsl:if test="$volume/cleanname">
      <xsl:text> - </xsl:text>
      <xsl:value-of select="$volume/cleanname"/>
    </xsl:if>

    <xsl:if test="$volume/filesystem">
      
      <xsl:text> (</xsl:text>
      <xsl:value-of select="$volume/filesystem"/>
      <xsl:text>)</xsl:text>
    
    </xsl:if>

    <xsl:choose>
      
      <xsl:when test="$volume/type">
        <xsl:text> [</xsl:text>
        <xsl:value-of 
          select="$strings/volume/types/value[@key = $volume/type]"/>
        <xsl:text>]</xsl:text>
      </xsl:when>
      
      <xsl:otherwise/>
    </xsl:choose>
    
    <xsl:text> </xsl:text>
    <xsl:apply-templates select="$volume/size" mode="units"/>

    <xsl:if test="$volume/used &gt; 0">
      <xsl:text> </xsl:text>
      <xsl:apply-templates select="$volume/used" mode="units"/>
    </xsl:if>
      
    <xsl:value-of select="$newline"/>
      
    <xsl:if test="$volume/volumes/device">
      <xsl:call-template name="volumes">
        <xsl:with-param name="indent" select="$indent + 2"/>
        <xsl:with-param name="volumes" select="$volume/volumes"/>
      </xsl:call-template>
    </xsl:if>
    
  </xsl:template>

  <!-- Storage volume. -->
  <xsl:template match="volume">
  
    <xsl:variable name="type" select="type"/>

    <xsl:if test="position() &gt; 1">
      <xsl:value-of select="$newline"/>
    </xsl:if>
    
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>

    <xsl:value-of select="device"/>
    
    <xsl:if test="cleanname">
      <xsl:text> - </xsl:text>
      <xsl:value-of select="cleanname"/>
    </xsl:if>
    
    <xsl:if test="type">
      <xsl:text> [</xsl:text>
      <xsl:value-of select="$strings/volume/types/value[@key = $type]"/>
      <xsl:text>] </xsl:text>
    </xsl:if>
  
    <xsl:text> </xsl:text>
    <xsl:apply-templates select="size" mode="units"/>
    
    <xsl:if test="free != 0">
      <xsl:text> (</xsl:text>
      <xsl:apply-templates select="free" mode="units"/>
      <xsl:text> </xsl:text>
      <xsl:value-of select="$strings/volume/free"/>
      <xsl:text>)</xsl:text>
    </xsl:if>

    <xsl:value-of select="$newline"/>
    
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>

    <xsl:if test="filesystem">      
      <xsl:value-of select="filesystem"/>
      <xsl:value-of select="$newline"/>
    </xsl:if>
    
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>

    <xsl:choose>
      <xsl:when test="mountpoint">
        <xsl:value-of select="$strings/volume/mountpoint"/>
        <xsl:value-of select="mountpoint"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$strings/volume/not_mounted"/>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:value-of select="$newline"/>
    
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>

    <xsl:if test="encryption">
      <xsl:variable 
        name="encryptionstatus" select="encryption/status"/>
      
      <xsl:value-of 
        select="$strings/volume/encryption/value[@key = $encryptionstatus]"/>
        
      <xsl:if test="encryption/status != 'encrypted'">
        <xsl:text> </xsl:text>
        <xsl:value-of select="encryption/progress"/>
        <xsl:value-of select="$strings/volume/percent_done"/>
      </xsl:if>
      <xsl:value-of select="$newline"/>
    </xsl:if>
    
    <xsl:if test="errors">
      <xsl:call-template name="plural">
        <xsl:with-param name="keys" select="$strings/volume"/>
        <xsl:with-param name="key">
          <xsl:text>errors</xsl:text>
        </xsl:with-param>
        <xsl:with-param name="value" select="errors"/>
      </xsl:call-template>
      <xsl:value-of select="$newline"/>
    </xsl:if>
    
    <xsl:if test="volumes/device">
      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="2"/>
      </xsl:call-template>
      <xsl:value-of select="$strings/volume/partitions"/>
      <xsl:value-of select="$newline"/>
      <xsl:call-template name="volumes">
        <xsl:with-param name="indent" select="$indent + 2"/>
        <xsl:with-param name="volumes" select="volumes"/>
      </xsl:call-template>
    </xsl:if>

  </xsl:template>

  <!-- Network information. -->
  <xsl:template match="network">
  
    <xsl:call-template name="header"/>

    <xsl:for-each select="interfaces/networkinterface">
      <xsl:variable name="interface" select="interface"/>
      <xsl:variable 
        name="modes" 
        select="/etrecheck/hardware/wireless[name = $interface]"/>

      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="2"/>
      </xsl:call-template>

      <xsl:value-of select="$strings/network/interface"/>
      <xsl:value-of select="interface"/>
      <xsl:value-of select="$strings/network/colon"/>
      <xsl:value-of select="name"/>
      <xsl:value-of select="$newline"/>
      
      <xsl:if test="$modes">
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="4"/>
        </xsl:call-template>
        <xsl:value-of select="$modes/modes"/>
        <xsl:value-of select="$newline"/>
      </xsl:if>

      <xsl:if test="count(ipv4addresses/address) &gt; 0">
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="4"/>
        </xsl:call-template>
        <xsl:call-template name="plural">
          <xsl:with-param name="keys" select="$strings/network"/>
          <xsl:with-param name="key">
            <xsl:text>ipv4address</xsl:text>
          </xsl:with-param>
          <xsl:with-param 
            name="value" 
            select="exslt:node-set(count(ipv4addresses/address))"/>
        </xsl:call-template>
        <xsl:value-of select="$newline"/>
      </xsl:if>
      
      <xsl:if test="count(ipv6addresses/address) &gt; 0">
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="4"/>
        </xsl:call-template>
        <xsl:call-template name="plural">
          <xsl:with-param name="keys" select="$strings/network"/>
          <xsl:with-param name="key">
            <xsl:text>ipv6address</xsl:text>
          </xsl:with-param>
          <xsl:with-param 
            name="value" 
            select="exslt:node-set(count(ipv6addresses/address))"/>
        </xsl:call-template>
        <xsl:value-of select="$newline"/>
      </xsl:if>
      
      <xsl:if test="proxyautodiscovery = 'true'">
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="4"/>
        </xsl:call-template>
        <xsl:value-of select="$strings/network/proxyautodiscovery"/>
        <xsl:value-of select="$newline"/>
      </xsl:if>
      
      <xsl:if test="proxyautoconfig = 'true'">
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="4"/>
        </xsl:call-template>
        <xsl:value-of select="$strings/network/proxyautoconfig"/>
        <xsl:text> </xsl:text>
        <xsl:value-of select="$strings/network/proxyautoconfigurl"/>
        <xsl:value-of select="$newline"/>
      </xsl:if>

      <xsl:if test="proxies/proxy">
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="4"/>
        </xsl:call-template>
        <xsl:value-of select="$strings/network/proxies"/>
        <xsl:for-each select="proxies/proxy">
          <xsl:if test="position() &gt; 1">
            <xsl:text>, </xsl:text>
          </xsl:if>
          
          <xsl:value-of select="type"/>
        </xsl:for-each>
        <xsl:value-of select="$newline"/>
      </xsl:if>

    </xsl:for-each>
    
    <xsl:if test="icloudfree">
      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="2"/>
      </xsl:call-template>
      <xsl:value-of select="$strings/hardware/icloudfree"/>
      <xsl:apply-templates select="icloudfree" mode="units"/>
      <xsl:value-of select="$strings/hardware/available"/>
      <xsl:value-of select="$newline"/>
    </xsl:if>

    <xsl:if test="count(icloudpendingfiles//file) &gt; 0">
      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="2"/>
      </xsl:call-template>
      <xsl:value-of select="$strings/hardware/icloudstatus"/>
      <xsl:call-template name="plural">
        <xsl:with-param name="keys" select="$strings/hardware"/>
        <xsl:with-param name="key">
          <xsl:text>icloudpendingfiles</xsl:text>
        </xsl:with-param>
        <xsl:with-param 
          name="value" 
          select="exslt:node-set(count(icloudpendingfiles//file))"/>
      </xsl:call-template>
      <xsl:value-of select="$newline"/>
    </xsl:if>

    <xsl:value-of select="$newline"/>
    
  </xsl:template>
  
  <!-- System software. -->
  <xsl:template match="systemsoftware">
  
    <xsl:call-template name="header"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>

    <xsl:value-of select="name"/>
    <xsl:text> </xsl:text>
    <xsl:value-of select="version"/>
    <xsl:text> (</xsl:text>
    <xsl:value-of select="build"/>
    <xsl:text>) </xsl:text>
    <xsl:value-of select="$newline"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>

    <xsl:value-of select="$strings/uptime"/>
    <xsl:apply-templates select="uptime"/>
    <xsl:value-of select="$newline"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>

    <xsl:call-template name="format">
      <xsl:with-param name="format" 
        select="$strings/systemsoftware/systemload"/>
      <xsl:with-param name="value" select="load1|load5|load15"/>
    </xsl:call-template>

    <xsl:value-of select="$newline"/>
    <xsl:value-of select="$newline"/>

  </xsl:template>

  <!-- Uptime. -->
  <xsl:template match="uptime">

    <xsl:variable name="days" select=". div 24"/>
    <xsl:variable name="hours" select=". mod 24"/>
    
    <xsl:choose>
    
      <xsl:when test="$days &gt;= 1">
        <xsl:call-template name="plural">
          <xsl:with-param name="keys" select="$strings/uptime_human"/>
          <xsl:with-param name="key" select="'day'"/>
          <xsl:with-param name="value" select="$days"/>
        </xsl:call-template>
      </xsl:when>
      
      <xsl:otherwise>
        <xsl:call-template name="plural">
          <xsl:with-param name="keys" select="$strings/uptime_human"/>
          <xsl:with-param name="key" select="'hour'"/>
          <xsl:with-param name="value" select="$hours"/>
        </xsl:call-template>
      </xsl:otherwise>
      
    </xsl:choose>
    
  </xsl:template>

  <!-- Configuration files. -->
  <xsl:template match="configurationfiles">
  
    <xsl:if test="count(*)">
      <xsl:call-template name="header"/>
      
      <xsl:if test="unexpectedsudoerssize">
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>
        <xsl:value-of select="$strings/unexpectedsudoerssize"/>
        <xsl:text> </xsl:text>
        <xsl:apply-templates select="size" mode="units"/>
        <xsl:text> </xsl:text>
        <xsl:value-of select="$strings/but_expected"/>
        <xsl:text> </xsl:text>
        <xsl:apply-templates select="expectedsize" mode="units"/>
        <xsl:value-of select="$newline"/>
      </xsl:if>
      <xsl:if test="etcsysctlconfexists">
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>
        <xsl:value-of select="$strings/etcsysctlconfexists"/>
        <xsl:value-of select="$newline"/>
      </xsl:if>
      <xsl:if test="etclaunchdconfexists">
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>
        <xsl:value-of select="$strings/etclaunchdconfexists"/>
        <xsl:value-of select="$newline"/>
      </xsl:if>
      <xsl:if test="hostscount">
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>
        <xsl:value-of select="$strings/hostscount"/>
        <xsl:value-of select="hostscount"/>
        
        <xsl:if test="hostscorrupt">
          <xsl:text> - </xsl:text>
          <xsl:value-of select="$strings/hostscorrupt"/>
        </xsl:if>
        <xsl:value-of select="$newline"/>
      </xsl:if>
      
      <xsl:value-of select="$newline"/>
    </xsl:if>
        
  </xsl:template>

  <!-- Security information. -->
  <xsl:template match="security">
  
    <xsl:call-template name="header"/>
    
    <xsl:variable name="systemstatuswidth">
      <xsl:call-template name="maxwidth">
        <xsl:with-param 
          name="nodes" 
          select="$strings/security/system | $strings/security/gatekeeper | $strings/security/sip"/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    <xsl:call-template name="rpad">
      <xsl:with-param name="string" select="$strings/security/system"/>
      <xsl:with-param name="width" select="$systemstatuswidth + 2"/>
    </xsl:call-template>
          
    <xsl:value-of select="$strings/security/status"/>
    <xsl:value-of select="$newline"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    <xsl:call-template name="rpad">
      <xsl:with-param name="string" select="$strings/security/gatekeeper"/>
      <xsl:with-param name="width" select="$systemstatuswidth + 2"/>
    </xsl:call-template>
          
    <xsl:variable name="gatekeeperstatus" select="gatekeeper/status"/>
    
    <xsl:value-of 
      select="$strings/security/gatekeeper_setting/value[@key = $gatekeeperstatus]"/>
    
    <xsl:value-of select="$newline"/>

    <xsl:if test="/etrecheck/systemsoftware/majorversion &gt; 10">

      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="2"/>
      </xsl:call-template>
      <xsl:call-template name="rpad">
        <xsl:with-param name="string" select="$strings/security/sip"/>
        <xsl:with-param name="width" select="$systemstatuswidth + 2"/>
      </xsl:call-template>
            
      <xsl:choose>
        <xsl:when test="SIP = 'enabled'">
          <xsl:value-of select="$strings/security/sip_enabled"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="SIP"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:if>
    
    <xsl:value-of select="$newline"/>
      
    <xsl:value-of select="$newline"/>

    <xsl:variable name="systemwidth">
      <xsl:call-template name="maxwidth">
        <xsl:with-param 
          name="nodes" 
          select="$strings/security/system | $strings/security/gatekeeper | $strings/security/xprotect | $strings/security/mrt"/>
      </xsl:call-template>
    </xsl:variable>
    
    <xsl:variable name="latestwidth">10</xsl:variable>

    <xsl:variable name="installedwidth">10</xsl:variable>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    <xsl:call-template name="rpad">
      <xsl:with-param name="string"/>
      <xsl:with-param name="width" select="$systemwidth + 2"/>
    </xsl:call-template>

    <xsl:call-template name="center">
      <xsl:with-param name="string" select="$strings/security/versions"/>
      <xsl:with-param 
        name="width" select="$latestwidth + 2 + $installedwidth"/>
    </xsl:call-template>
    
    <xsl:value-of select="$newline"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    <xsl:call-template name="rpad">
      <xsl:with-param name="string" select="$strings/security/system"/>
      <xsl:with-param name="width" select="$systemwidth + 2"/>
    </xsl:call-template>
          
    <xsl:call-template name="rpad">
      <xsl:with-param 
        name="string" select="$strings/security/currentversion"/>
        
      <xsl:with-param name="width" select="$latestwidth + 2"/>
    </xsl:call-template>

    <xsl:call-template name="rpad">
      <xsl:with-param 
        name="string" select="$strings/security/installedversion"/>
        
      <xsl:with-param name="width" select="$installedwidth + 2"/>
    </xsl:call-template>

    <xsl:value-of select="$newline"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    <xsl:call-template name="rpad">
      <xsl:with-param name="string" select="$strings/security/gatekeeper"/>
      <xsl:with-param name="width" select="$systemwidth + 2"/>
    </xsl:call-template>
          
    <xsl:call-template name="rpad">
      <xsl:with-param 
        name="string" select="substring(gatekeeper/currentversion, 1, 10)"/>
        
      <xsl:with-param name="width" select="$latestwidth + 2"/>
    </xsl:call-template>

    <xsl:call-template name="rpad">
      <xsl:with-param 
        name="string" 
        select="substring(gatekeeper/installedversion, 1, 10)"/>
        
      <xsl:with-param name="width" select="$installedwidth + 2"/>
    </xsl:call-template>

    <xsl:value-of select="$newline"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    <xsl:call-template name="rpad">
      <xsl:with-param name="string" select="$strings/security/xprotect"/>
      <xsl:with-param name="width" select="$systemwidth + 2"/>
    </xsl:call-template>
          
    <xsl:call-template name="rpad">
      <xsl:with-param 
        name="string" select="substring(xprotect/currentversion, 1, 10)"/>
        
      <xsl:with-param name="width" select="$latestwidth + 2"/>
    </xsl:call-template>

    <xsl:call-template name="rpad">
      <xsl:with-param 
        name="string" select="substring(xprotect/installedversion, 1, 10)"/>
        
      <xsl:with-param name="width" select="$installedwidth + 2"/>
    </xsl:call-template>

    <xsl:value-of select="$newline"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    <xsl:call-template name="rpad">
      <xsl:with-param name="string" select="$strings/security/mrt"/>
      <xsl:with-param name="width" select="$systemwidth + 2"/>
    </xsl:call-template>
          
    <xsl:call-template name="rpad">
      <xsl:with-param 
        name="string" select="substring(mrt/currentversion, 1, 10)"/>
        
      <xsl:with-param name="width" select="$latestwidth + 2"/>
    </xsl:call-template>

    <xsl:if test="/etrecheck/systemsoftware/majorversion &gt; 10">
      <xsl:call-template name="rpad">
        <xsl:with-param 
          name="string" select="substring(mrt/installedversion, 1, 10)"/>
          
        <xsl:with-param name="width" select="$installedwidth + 2"/>
      </xsl:call-template>

      <xsl:value-of select="$newline"/>

      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="2"/>
      </xsl:call-template>
    </xsl:if>
    
    <xsl:value-of select="$newline"/>

  </xsl:template>

  <!-- Adware information. -->
  <xsl:template match="adware">
  
    <xsl:if test="count(launchdfiles/launchdfile) &gt; 0 or count(safariextensions/extension) &gt; 0">
      <xsl:call-template name="header"/>
      
      <xsl:for-each select="launchdfiles/launchdfile">

        <xsl:variable name="adware" select="adware"/>

        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>
        <xsl:value-of select="$strings/adware/launchd"/>
        <xsl:value-of select="path_safe"/>
        <xsl:value-of select="$newline"/>
        
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="4"/>
        </xsl:call-template>
        <xsl:value-of select="$strings/adware/executable"/>
        <xsl:value-of select="executable_safe"/>
        <xsl:for-each select="arguments_safe/argument">
          <xsl:value-of select="concat(' ', .)"/>
        </xsl:for-each>
        <xsl:value-of select="$newline"/>

        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="4"/>
        </xsl:call-template>
        <xsl:value-of select="$strings/adware/reason"/>
        <xsl:value-of 
          select="$strings/adware/reason_text/value[@key = $adware]"/>
        <xsl:value-of select="$newline"/>
      </xsl:for-each>

      <xsl:for-each select="safariextensions/extension">

        <xsl:variable name="adware" select="adware"/>

        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>
        <xsl:value-of select="$strings/adware/safariextension"/>
        <xsl:value-of select="displayname"/>
        <xsl:value-of select="$newline"/>
          
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="4"/>
        </xsl:call-template>
        <xsl:value-of select="$strings/adware/reason"/>
        <xsl:value-of 
          select="$strings/adware/reason_text/value[@key = $adware]"/>
        <xsl:value-of select="$newline"/>
      </xsl:for-each>

      <xsl:value-of select="$newline"/>

    </xsl:if>
  
  </xsl:template>

  <!-- Unsigned files information. -->
  <xsl:template match="unsigned">
  
    <xsl:if test="count(launchdfiles/launchdfile) &gt; 0 or count(safariextensions/extension) &gt; 0">
      <xsl:call-template name="header"/>
      
      <xsl:for-each select="launchdfiles/launchdfile">

        <xsl:variable name="details" select="details"/>
        
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>
        <xsl:value-of select="$strings/unsigned/launchd"/>
        <xsl:value-of select="path_safe"/>
        <xsl:value-of select="$newline"/>
        
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="4"/>
        </xsl:call-template>
        <xsl:value-of select="$strings/unsigned/executable"/>
        <xsl:value-of select="executable_safe"/>
        <xsl:for-each select="arguments_safe/argument">
          <xsl:text> </xsl:text>
          <xsl:value-of select="."/>
        </xsl:for-each>
        <xsl:value-of select="$newline"/>
        
        <xsl:if test="details">
          <xsl:call-template name="indent">
            <xsl:with-param name="indent" select="4"/>
          </xsl:call-template>
          <xsl:value-of select="$strings/unsigned/details"/>
          <xsl:value-of 
            select="$strings/unsigned/details_text/value[@key = $details]"/>
          <xsl:value-of select="$newline"/>
        </xsl:if>
        
      </xsl:for-each>

      <xsl:for-each select="safariextensions/extension">

        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="4"/>
        </xsl:call-template>
        <xsl:value-of select="$strings/adware/safariextension"/>
        <xsl:value-of select="displayname"/>
        <xsl:value-of select="$newline"/>
      </xsl:for-each>

      <xsl:value-of select="$newline"/>

    </xsl:if>
        
  </xsl:template>

  <!-- Clean up information. -->
  <xsl:template match="cleanup">
  
    <xsl:variable 
      name="missingexecutables" 
      select="count(launchdfiles//executable) &gt; 0"/>
    
    <xsl:variable 
      name="missingextensions" 
      select="count(safariextensions//path) &gt; 0"/>

    <xsl:variable 
      name="notificationspam" 
      select="count(notificationspam/spammer) &gt; 0"/>

    <xsl:if test="$missingexecutables or $missingextensions or $notificationspam">
      <xsl:call-template name="header"/>

      <xsl:for-each select="launchdfiles/launchdfile">
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>
        <xsl:value-of select="path_safe"/>
        <xsl:value-of select="$newline"/>

        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="4"/>
        </xsl:call-template>
        <xsl:value-of select="executable_safe"/>
        <xsl:value-of select="$newline"/>

        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="4"/>
        </xsl:call-template>
        <xsl:value-of select="$strings/executable_not_found"/>
        <xsl:value-of select="$newline"/>
      </xsl:for-each>

      <xsl:if test="$missingexecutables and $missingextensions">
        <xsl:value-of select="$newline"/>
      </xsl:if>
      
      <xsl:for-each select="safariextensions/extension">
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>
        <xsl:value-of select="displayname"/>
        <xsl:text> - </xsl:text>
        <xsl:value-of select="$strings/extension_not_found"/>
        <xsl:value-of select="$newline"/>
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="4"/>
        </xsl:call-template>
        <xsl:value-of select="path_safe"/>
        <xsl:value-of select="$newline"/>
      </xsl:for-each>

      <xsl:if test="($missingexecutables or $missingextensions) and $notificationspam">
        <xsl:value-of select="$newline"/>
      </xsl:if>
      
      <xsl:for-each select="notificationspam/spammer">
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>
        <xsl:value-of select="name"/>
        <xsl:text> - </xsl:text>
        <xsl:call-template name="plural">
          <xsl:with-param name="key">spamnotificationcount</xsl:with-param>
          <xsl:with-param name="value">
            <xsl:value-of select="count"/>
          </xsl:with-param>
        </xsl:call-template>
        <xsl:value-of select="$newline"/>
      </xsl:for-each>

      <xsl:value-of select="$newline"/>
    </xsl:if>
        
  </xsl:template>

  <!-- Kernel extensions. -->
  <xsl:template match="kernelextensions">
  
    <xsl:call-template name="header"/>
    
    <xsl:for-each select="directory">
      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="2"/>
      </xsl:call-template>
      <xsl:value-of select="path_safe"/>
      <xsl:value-of select="$newline"/>
      
      <xsl:for-each select="extensions/extension">
        <xsl:variable name="status" select="status"/>
    
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="4"/>
        </xsl:call-template>
        <xsl:call-template name="rpad">
          <xsl:with-param name="string">
            <xsl:text>[</xsl:text>
            <xsl:value-of select="$strings/kernelextensions/status/value[@key = $status]"/>
            <xsl:text>] </xsl:text>
          </xsl:with-param>
          <xsl:with-param name="width" select="$kernelextensionstatuswidth + 3"/>
        </xsl:call-template>

        <xsl:value-of select="filename"/>
        <xsl:text> (</xsl:text>
        <xsl:value-of select="version"/>
        
        <xsl:if test="osversion">
          <xsl:text> - </xsl:text>
          <xsl:value-of select="osversion"/>
        </xsl:if>
        
        <xsl:text>)</xsl:text>
        <xsl:value-of select="$newline"/>
      </xsl:for-each>
    </xsl:for-each>
        
    <xsl:value-of select="$newline"/>

  </xsl:template>

  <!-- Print startup items. -->
  <xsl:template match="startupitems">
  
    <xsl:if test="count(startupitem) &gt; 0">
      <xsl:call-template name="header"/>
      
      <xsl:for-each select="startupitem">
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>
        <xsl:value-of select="name"/>
        <xsl:text> </xsl:text>
        <xsl:value-of select="$strings/path"/>
        <xsl:value-of select="path"/>
        <xsl:text> </xsl:text>
        <xsl:value-of select="version"/>
        <xsl:value-of select="$newline"/>
      </xsl:for-each>
      
      <xsl:value-of select="$newline"/>

    </xsl:if>
        
  </xsl:template>

  <!-- Print system launch agents. -->
  <xsl:template match="systemlaunchagents">
  
    <xsl:call-template name="header"/>
    
    <xsl:apply-templates select="applelaunchdfile"/>
    
    <xsl:value-of select="$newline"/>

  </xsl:template>
  
  <!-- Print system launch daemons. -->
  <xsl:template match="systemlaunchdaemons">
  
    <xsl:call-template name="header"/>
    
    <xsl:apply-templates select="applelaunchdfile"/>
        
    <xsl:value-of select="$newline"/>

  </xsl:template>

  <!-- Print Apple tasks. -->
  <xsl:template match="applelaunchdfile">
    
    <xsl:variable name="status" select="status"/>
    
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    <xsl:call-template name="rpad">
      <xsl:with-param name="string">
        <xsl:text>[</xsl:text>
        <xsl:value-of select="$strings/launchd/status/value[@key = $status]"/>
        <xsl:text>] </xsl:text>
      </xsl:with-param>
      <xsl:with-param name="width" select="$launchdstatuswidth + 3"/>
    </xsl:call-template>
    
    <xsl:text> </xsl:text>
    <xsl:call-template name="plural">
      <xsl:with-param name="keys" select="$strings"/>
      <xsl:with-param name="key">
        <xsl:text>apple_tasks</xsl:text>
      </xsl:with-param>
      <xsl:with-param name="value" select="count"/>
    </xsl:call-template>
    <xsl:value-of select="$newline"/>
        
  </xsl:template>

  <!-- Print launch agents. -->
  <xsl:template match="launchagents">
  
    <xsl:if test="count(launchdfile) &gt; 0">
      <xsl:call-template name="header"/>
      
      <xsl:apply-templates select="launchdfile"/>
      
      <xsl:value-of select="$newline"/>
    
    </xsl:if>
        
  </xsl:template>

  <!-- Print launch daemons. -->
  <xsl:template match="launchdaemons">
  
    <xsl:if test="count(launchdfile) &gt; 0">
      <xsl:call-template name="header"/>
      
      <xsl:apply-templates select="launchdfile"/>

      <xsl:value-of select="$newline"/>
    
    </xsl:if>
        
  </xsl:template>

  <!-- Print user launch agents. -->
  <xsl:template match="userlaunchagents">
  
    <xsl:if test="count(launchdfile) &gt; 0">
      <xsl:call-template name="header"/>
      
      <xsl:apply-templates select="launchdfile"/>

      <xsl:value-of select="$newline"/>

    </xsl:if>
        
  </xsl:template>

  <!-- Print a launchd task. -->
  <xsl:template match="launchdfile">
  
    <xsl:variable name="status" select="status"/>
    
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    <xsl:call-template name="rpad">
      <xsl:with-param name="string">
        <xsl:text>[</xsl:text>
        <xsl:value-of select="$strings/launchd/status/value[@key = $status]"/>
        <xsl:text>] </xsl:text>
      </xsl:with-param>
      <xsl:with-param name="width" select="$launchdstatuswidth + 3"/>
    </xsl:call-template>

    <xsl:value-of select="filename"/>
    <xsl:text> (</xsl:text>
    <xsl:value-of select="author"/>
    
    <xsl:choose>
    
      <xsl:when test="author = 'Shell Script'">
        <xsl:text> </xsl:text>
        <xsl:value-of select="execrc"/>
      </xsl:when>
      
      <xsl:when test="author = '?'">
        <xsl:text> </xsl:text>
        <xsl:value-of select="plistcrc"/>
        <xsl:text> </xsl:text>
        <xsl:value-of select="execrc"/>
      </xsl:when>
      
      <xsl:otherwise>
      </xsl:otherwise>
    
    </xsl:choose>
    
    <xsl:text> - </xsl:text>
    <xsl:value-of select="$strings/installed"/>
    <xsl:text> </xsl:text>
    <xsl:apply-templates select="installdate"/>
    <xsl:text>)</xsl:text>

    <xsl:value-of select="$newline"/>

  </xsl:template>

  <!-- Print login items. -->
  <xsl:template match="loginitems">
  
    <xsl:if test="count(loginitem) &gt; 0">
      <xsl:call-template name="header"/>
      
      <xsl:apply-templates select="loginitem"/>

      <xsl:value-of select="$newline"/>
    
    </xsl:if>
        
  </xsl:template>

  <!-- Print a single login item. -->
  <xsl:template match="loginitem">
  
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    <xsl:value-of select="name"/>
    <xsl:text> </xsl:text>
    <xsl:value-of select="type"/>
    
    <xsl:if test="signature or installdate">
    
      <xsl:text> (</xsl:text>
      
      <xsl:choose>
      
        <xsl:when test="signature = 'none'">
        </xsl:when>
        
        <xsl:when test="signature">
          <xsl:value-of select="signature"/>
        </xsl:when>

        <xsl:otherwise>
        </xsl:otherwise>
      
      </xsl:choose>
      
      <xsl:if test="installdate">

        <xsl:choose>
        
          <xsl:when test="signature = 'none'">
          </xsl:when>
          
          <xsl:when test="signature">
            <xsl:text> - </xsl:text>
          </xsl:when>

          <xsl:otherwise>
          </xsl:otherwise>
        
        </xsl:choose>

        <xsl:value-of select="$strings/installed"/>
        <xsl:text> </xsl:text>
        <xsl:apply-templates select="installdate"/>
        <xsl:text>)</xsl:text>
      </xsl:if>
      
    </xsl:if>
    <xsl:value-of select="$newline"/>
    
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="4"/>
    </xsl:call-template>
    <xsl:text> (</xsl:text>
    <xsl:value-of select="path_safe"/>
    <xsl:text>)</xsl:text>
    <xsl:value-of select="$newline"/>
        
  </xsl:template>

  <!-- Print internet plugins. -->
  <xsl:template match="internetplugins">
  
    <xsl:if test="count(plugin) &gt; 0">
      <xsl:call-template name="header"/>

      <xsl:apply-templates select="plugin"/>
    
      <xsl:value-of select="$newline"/>
    
    </xsl:if>
        
  </xsl:template>

  <!-- Print user internet plugins. -->
  <xsl:template match="userinternetplugins">
  
    <xsl:if test="count(plugin) &gt; 0">
      <xsl:call-template name="header"/>

      <xsl:apply-templates select="plugin"/>
    
      <xsl:value-of select="$newline"/>
    
    </xsl:if>
        
  </xsl:template>

  <!-- Print audio plugins. -->
  <xsl:template match="audioplugins">
  
    <xsl:if test="count(plugin) &gt; 0">
      <xsl:call-template name="header"/>

      <xsl:apply-templates select="plugin"/>
    
      <xsl:value-of select="$newline"/>
    
    </xsl:if>
        
  </xsl:template>

  <!-- Print user audio plugins. -->
  <xsl:template match="useraudioplugins">
  
    <xsl:if test="count(plugin) &gt; 0">
      <xsl:call-template name="header"/>

      <xsl:apply-templates select="plugin"/>
    
      <xsl:value-of select="$newline"/>
    
    </xsl:if>
        
  </xsl:template>

  <!-- Print iTunes plugins. -->
  <xsl:template match="itunesplugins">
  
    <xsl:if test="count(plugin) &gt; 0">
      <xsl:call-template name="header"/>

      <xsl:apply-templates select="plugin"/>
    
      <xsl:value-of select="$newline"/>
    
    </xsl:if>
        
  </xsl:template>

  <!-- Print user iTunes plugins. -->
  <xsl:template match="useritunesplugins">
  
    <xsl:if test="count(plugin) &gt; 0">
      <xsl:call-template name="header"/>

      <xsl:apply-templates select="plugin"/>
    
      <xsl:value-of select="$newline"/>
    
    </xsl:if>
        
  </xsl:template>

  <!-- Print a single plugin. -->
  <xsl:template match="plugin">
  
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    <xsl:value-of select="name"/>
    <xsl:text>: </xsl:text>
    <xsl:value-of select="version"/>
    
    <xsl:if test="installdate">
      <xsl:text> (</xsl:text>
      <xsl:value-of select="$strings/installed"/>
      <xsl:text> </xsl:text>
      <xsl:apply-templates select="installdate"/>
      <xsl:text>)</xsl:text>
    </xsl:if>
    <xsl:value-of select="$newline"/>
        
  </xsl:template>

  <!-- Print Safari extensions. -->
  <xsl:template match="safariextensions">
  
    <xsl:if test="count(extension) &gt; 0">
      <xsl:call-template name="header"/>

      <xsl:apply-templates select="extension"/>
    
      <xsl:value-of select="$newline"/>
    
    </xsl:if>
        
  </xsl:template>
  
  <!-- Print a single Safari extension. -->
  <xsl:template match="extension">
  
    <xsl:variable name="status" select="status"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    <xsl:value-of select="name"/>
    <xsl:text> - </xsl:text>
    <xsl:value-of select="developer"/>
    <xsl:text> - </xsl:text>
    <a target="_blank">
      <xsl:attribute name="href">
        <xsl:value-of select="url"/>
      </xsl:attribute>
      <strong>
        <xsl:value-of select="url"/>
      </strong>
    </a>
    <xsl:text> (</xsl:text>
    <xsl:value-of select="$strings/installed"/>
    <xsl:text> </xsl:text>
    <xsl:apply-templates select="installdate"/>
    <xsl:text>)</xsl:text>
    <xsl:value-of select="$newline"/>
         
  </xsl:template>

  <!-- Print Preference panes. -->
  <xsl:template match="preferencepanes">
  
    <xsl:if test="count(preferencepane) &gt; 0">
      <xsl:call-template name="header"/>

      <xsl:apply-templates select="preferencepane"/>

      <xsl:value-of select="$newline"/>
    </xsl:if>
        
  </xsl:template>
  
  <!-- Print a single Preference pane. -->
  <xsl:template match="preferencepane">
  
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    <xsl:value-of select="name"/>
    <xsl:text> (</xsl:text>
    <xsl:value-of select="$strings/installed"/>
    <xsl:text> </xsl:text>
    <xsl:apply-templates select="installdate"/>
    <xsl:text>)</xsl:text>
    <xsl:value-of select="$newline"/>
        
  </xsl:template>

  <!-- Print Fonts. -->
  <xsl:template match="fonts">
  
    <!-- This should only be bad fonts. -->
    <xsl:if test="count(fonts) &gt; 0">
      <xsl:call-template name="header"/>

      <xsl:apply-templates select="font"/>

      <xsl:value-of select="$newline"/>
    </xsl:if>
        
  </xsl:template>
  
  <!-- Print a single font. -->
  <xsl:template match="font">
  
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    <xsl:text>[</xsl:text>
    <xsl:value-of select="@status"/>
    <xsl:text>] </xsl:text>
    <xsl:value-of select="name"/>
    <xsl:text> - </xsl:text>
    <xsl:value-of select="date"/>
    <xsl:text>)</xsl:text>
    <xsl:value-of select="$newline"/>
        
  </xsl:template>

  <!-- Print Time Machine information. -->
  <xsl:template match="timemachine">
  
    <xsl:call-template name="header"/>

    <xsl:choose>
      <xsl:when test="notconfigured = 'true'">
      
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>
        <xsl:value-of select="$strings/tm/notconfigured"/>
        <xsl:value-of select="$newline"/>
        
      </xsl:when>
      
      <xsl:otherwise>
      
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>
        <xsl:value-of select="$strings/tm/skip_system_files"/>
        <xsl:apply-templates select="skipsystemfiles"/>
        <xsl:value-of select="$newline"/>
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>
        <xsl:value-of select="$strings/tm/mobile_backups"/>
        <xsl:apply-templates select="mobilebackups"/>
        <xsl:value-of select="$newline"/>
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>
        <xsl:value-of select="$strings/tm/auto_backup"/>
        <xsl:apply-templates select="autobackup"/>
        <xsl:value-of select="$newline"/>
        
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>
        <xsl:value-of select="$strings/tm/volumes_backed_up"/>
        <xsl:value-of select="$newline"/>

        <xsl:apply-templates 
          select="volumesbeingbackedup/volume" mode="timemachine"/>
        
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>
        <xsl:value-of select="$strings/tm/destinations"/>
        <xsl:value-of select="$newline"/>

        <xsl:apply-templates select="destinations/destination"/>
        
      </xsl:otherwise>
   
    </xsl:choose>
    
    <xsl:value-of select="$newline"/>
  
  </xsl:template>

  <!-- Print a Time Machine volume. -->
  <xsl:template match="volume" mode="timemachine">
  
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="4"/>
    </xsl:call-template>
    <xsl:value-of select="cleanname"/>
    <xsl:text>: </xsl:text>
    <xsl:value-of select="$strings/tm/disk_size"/>
    <xsl:apply-templates select="size" mode="units"/>
    <xsl:text> - </xsl:text>
    <xsl:value-of select="$strings/tm/disk_used"/>
    <xsl:apply-templates select="used" mode="units"/>
    <xsl:text> </xsl:text>
    <xsl:value-of select="$newline"/>
      
  </xsl:template>

  <!-- Print a Time Machine destination. -->
  <xsl:template match="destination">
  
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="4"/>
    </xsl:call-template>
    <xsl:value-of select="cleanname"/>
    <xsl:text> [</xsl:text>
    <xsl:value-of select="type"/>
    <xsl:text>]</xsl:text>
    
    <xsl:if test="lastused = 'true'">
      <xsl:text> </xsl:text>
      <xsl:value-of select="$strings/tm/last_used"/>
    </xsl:if>
    <xsl:value-of select="$newline"/>
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="6"/>
    </xsl:call-template>
    <xsl:value-of select="$strings/tm/total_size"/>
    <xsl:apply-templates select="size" mode="units"/>
    <xsl:value-of select="$newline"/>
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="6"/>
    </xsl:call-template>
    <xsl:value-of select="$strings/tm/number_of_backups"/>
    <xsl:value-of select="count"/>
    <xsl:value-of select="$newline"/>
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="6"/>
    </xsl:call-template>
    <xsl:value-of select="$strings/tm/oldest_backup"/>
    <xsl:value-of select="oldestbackup"/>
    <xsl:value-of select="$newline"/>
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="6"/>
    </xsl:call-template>
    <xsl:value-of select="$strings/tm/last_backup"/>
    <xsl:value-of select="lastbackup"/>
    <xsl:value-of select="$newline"/>
    
  </xsl:template>

  <!-- Print a CPU usage information. -->
  <xsl:template match="cpu">
  
    <xsl:if test="count(process) &gt; 0">
      <xsl:call-template name="header"/>

        <xsl:variable name= "namewidth">
          <xsl:call-template name="maxprocessnamewidth">
            <xsl:with-param name="processes" select="process"/>
            <xsl:with-param name="header" select="$strings/cpu/process"/>
          </xsl:call-template>
        </xsl:variable>
        
        <xsl:variable name= "sourcewidth">
          <xsl:call-template name="maxprocesssourcewidth">
            <xsl:with-param name="processes" select="process"/>
            <xsl:with-param name="header" select="$strings/cpu/source"/>
          </xsl:call-template>
        </xsl:variable>

        <xsl:variable name= "valuewidth">
          <xsl:call-template name="maxwidth">
            <xsl:with-param 
              name="nodes" select="$strings/cpu/usage | process/usage"/>
          </xsl:call-template>
        </xsl:variable>

        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>
        <xsl:call-template name="rpad">
          <xsl:with-param name="string" select="$strings/cpu/process"/>
          <xsl:with-param name="width" select="$namewidth + 2"/>
        </xsl:call-template>

        <xsl:call-template name="rpad">
          <xsl:with-param name="string" select="$strings/cpu/source"/>
          <xsl:with-param name="width" select="$sourcewidth + 2"/>
        </xsl:call-template>

        <xsl:call-template name="lpad">
          <xsl:with-param name="string" select="$strings/cpu/usage"/>
          <xsl:with-param name="width" select="$valuewidth + 2"/>
        </xsl:call-template>
        <xsl:value-of select="$newline"/>

        <xsl:for-each select="process">
          <xsl:variable name="path" select="path"/>
          <xsl:variable 
            name="source" 
            select="/etrecheck/header/context/process[path = $path]/source"/>

          <xsl:call-template name="indent">
            <xsl:with-param name="indent" select="2"/>
          </xsl:call-template>
          
          <xsl:call-template name="rpad">
            <xsl:with-param name="string">
              <xsl:value-of select="name"/>
              <xsl:if test="count &gt; 1">
                <xsl:value-of select="concat(' (', count, ')')"/>
              </xsl:if>
            </xsl:with-param>
            <xsl:with-param name="width" select="$namewidth + 2"/>
          </xsl:call-template>
          
          <xsl:call-template name="rpad">
            <xsl:with-param name="string" select="$source"/>
            <xsl:with-param name="width" select="$sourcewidth + 2"/>
          </xsl:call-template>
          
          <xsl:variable name="value">
            <xsl:apply-templates select="usage" mode="units"/>
          </xsl:variable>
          
          <xsl:call-template name="lpad">
            <xsl:with-param name="string" select="$value"/>
            <xsl:with-param name="width" select="$valuewidth + 2"/>
          </xsl:call-template>
          
          <xsl:value-of select="$newline"/>

        </xsl:for-each>
      
      <xsl:value-of select="$newline"/>
      
    </xsl:if>
      
  </xsl:template>

  <!-- Print a memory usage information. -->
  <xsl:template match="memory">
  
    <xsl:if test="count(process) &gt; 0">
      <xsl:call-template name="header"/>

        <xsl:variable name= "namewidth">
          <xsl:call-template name="maxprocessnamewidth">
            <xsl:with-param name="processes" select="process"/>
            <xsl:with-param name="header" select="$strings/memory/process"/>
          </xsl:call-template>
        </xsl:variable>
        
        <xsl:variable name= "sourcewidth">
          <xsl:call-template name="maxprocesssourcewidth">
            <xsl:with-param name="processes" select="process"/>
            <xsl:with-param name="header" select="$strings/memory/source"/>
          </xsl:call-template>
        </xsl:variable>

        <xsl:variable name= "valuewidth">
          <xsl:call-template name="maxwidth">
            <xsl:with-param 
              name="nodes" select="$strings/memory/usage | process/size"/>
          </xsl:call-template>
        </xsl:variable>

        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>
        <xsl:call-template name="rpad">
          <xsl:with-param name="string" select="$strings/memory/process"/>
          <xsl:with-param name="width" select="$namewidth + 2"/>
        </xsl:call-template>

        <xsl:call-template name="rpad">
          <xsl:with-param name="string" select="$strings/memory/source"/>
          <xsl:with-param name="width" select="$sourcewidth + 2"/>
        </xsl:call-template>

        <xsl:call-template name="lpad">
          <xsl:with-param name="string" select="$strings/memory/usage"/>
          <xsl:with-param name="width" select="$valuewidth + 2"/>
        </xsl:call-template>
        <xsl:value-of select="$newline"/>

        <xsl:for-each select="process">
          <xsl:variable name="path" select="path"/>
          <xsl:variable 
            name="source" 
            select="/etrecheck/header/context/process[path = $path]/source"/>

          <xsl:call-template name="indent">
            <xsl:with-param name="indent" select="2"/>
          </xsl:call-template>
          
          <xsl:call-template name="rpad">
            <xsl:with-param name="string">
              <xsl:value-of select="name"/>
              <xsl:if test="count &gt; 1">
                <xsl:value-of select="concat(' (', count, ')')"/>
              </xsl:if>
            </xsl:with-param>
            <xsl:with-param name="width" select="$namewidth + 2"/>
          </xsl:call-template>
          
          <xsl:call-template name="rpad">
            <xsl:with-param name="string" select="$source"/>
            <xsl:with-param name="width" select="$sourcewidth + 2"/>
          </xsl:call-template>
          
          <xsl:variable name="value">
            <xsl:apply-templates select="size" mode="units"/>
          </xsl:variable>
          
          <xsl:call-template name="lpad">
            <xsl:with-param name="string" select="$value"/>
            <xsl:with-param name="width" select="$valuewidth + 2"/>
          </xsl:call-template>
          
          <xsl:value-of select="$newline"/>

        </xsl:for-each>
      
      <xsl:value-of select="$newline"/>

    </xsl:if>
      
  </xsl:template>

  <!-- Print a network usage information. -->
  <xsl:template match="networkusage">
  
    <xsl:if test="count(process) &gt; 0">
      <xsl:call-template name="header"/>

        <xsl:variable name= "namewidth">
          <xsl:call-template name="maxprocessnamewidth">
            <xsl:with-param name="processes" select="process"/>
            <xsl:with-param 
              name="header" select="$strings/networkusage/process"/>
          </xsl:call-template>
        </xsl:variable>
        
        <xsl:variable name= "sourcewidth">
          <xsl:call-template name="maxprocesssourcewidth">
            <xsl:with-param name="processes" select="process"/>
            <xsl:with-param 
              name="header" select="$strings/networkusage/source"/>
          </xsl:call-template>
        </xsl:variable>

        <xsl:variable name= "inputwidth">
          <xsl:call-template name="maxwidth">
            <xsl:with-param 
              name="nodes" 
              select="$strings/networkusage/input | process/inputsize"/>
          </xsl:call-template>
        </xsl:variable>

        <xsl:variable name= "outputwidth">
          <xsl:call-template name="maxwidth">
            <xsl:with-param 
              name="nodes" 
              select="$strings/networkusage/output | process/outputsize"/>
          </xsl:call-template>
        </xsl:variable>

        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>
        <xsl:call-template name="rpad">
          <xsl:with-param 
            name="string" select="$strings/networkusage/process"/>
          <xsl:with-param name="width" select="$namewidth + 2"/>
        </xsl:call-template>

        <xsl:call-template name="rpad">
          <xsl:with-param 
            name="string" select="$strings/networkusage/source"/>
          <xsl:with-param name="width" select="$sourcewidth + 2"/>
        </xsl:call-template>

        <xsl:call-template name="lpad">
          <xsl:with-param 
            name="string" select="$strings/networkusage/input"/>
          <xsl:with-param name="width" select="$inputwidth + 2"/>
        </xsl:call-template>

        <xsl:call-template name="lpad">
          <xsl:with-param 
            name="string" select="$strings/networkusage/output"/>
          <xsl:with-param name="width" select="$outputwidth + 2"/>
        </xsl:call-template>
        <xsl:value-of select="$newline"/>

        <xsl:for-each select="process">
          <xsl:variable name="path" select="path"/>
          <xsl:variable 
            name="source" 
            select="/etrecheck/header/context/process[path = $path]/source"/>

          <xsl:call-template name="indent">
            <xsl:with-param name="indent" select="2"/>
          </xsl:call-template>
          
          <xsl:call-template name="rpad">
            <xsl:with-param name="string">
              <xsl:value-of select="name"/>
              <xsl:if test="count &gt; 1">
                <xsl:value-of select="concat(' (', count, ')')"/>
              </xsl:if>
            </xsl:with-param>
            <xsl:with-param name="width" select="$namewidth + 2"/>
          </xsl:call-template>
          
          <xsl:call-template name="rpad">
            <xsl:with-param name="string" select="$source"/>
            <xsl:with-param name="width" select="$sourcewidth + 2"/>
          </xsl:call-template>
          
          <xsl:variable name="inputvalue">
            <xsl:apply-templates select="inputsize" mode="units"/>
          </xsl:variable>
          
          <xsl:call-template name="lpad">
            <xsl:with-param name="string" select="$inputvalue"/>
            <xsl:with-param name="width" select="$inputwidth + 2"/>
          </xsl:call-template>

          <xsl:variable name="outputvalue">
            <xsl:apply-templates select="outputsize" mode="units"/>
          </xsl:variable>
          
          <xsl:call-template name="lpad">
            <xsl:with-param name="string" select="$outputvalue"/>
            <xsl:with-param name="width" select="$outputwidth + 2"/>
          </xsl:call-template>

          <xsl:value-of select="$newline"/>

        </xsl:for-each>
      
      <xsl:value-of select="$newline"/>

    </xsl:if>
      
  </xsl:template>

  <!-- Print a energy usage information. -->
  <xsl:template match="energy">
  
    <xsl:if test="count(process) &gt; 0">
      <xsl:call-template name="header"/>

        <xsl:variable name= "namewidth">
          <xsl:call-template name="maxprocessnamewidth">
            <xsl:with-param name="processes" select="process"/>
            <xsl:with-param name="header" select="$strings/energy/process"/>
          </xsl:call-template>
        </xsl:variable>
        
        <xsl:variable name= "sourcewidth">
          <xsl:call-template name="maxprocesssourcewidth">
            <xsl:with-param name="processes" select="process"/>
            <xsl:with-param name="header" select="$strings/energy/source"/>
          </xsl:call-template>
        </xsl:variable>

        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>
        <xsl:call-template name="rpad">
          <xsl:with-param name="string" select="$strings/energy/process"/>
          <xsl:with-param name="width" select="$namewidth + 2"/>
        </xsl:call-template>

        <xsl:call-template name="rpad">
          <xsl:with-param name="string" select="$strings/energy/source"/>
          <xsl:with-param name="width" select="$sourcewidth + 2"/>
        </xsl:call-template>

        <xsl:value-of select="$strings/energy/usage"/>
        <xsl:value-of select="$newline"/>

        <xsl:for-each select="process">
          <xsl:variable name="path" select="path"/>
          <xsl:variable 
            name="source" 
            select="/etrecheck/header/context/process[path = $path]/source"/>

          <xsl:call-template name="indent">
            <xsl:with-param name="indent" select="2"/>
          </xsl:call-template>
          
          <xsl:call-template name="rpad">
            <xsl:with-param name="string">
              <xsl:value-of select="name"/>
              <xsl:if test="count &gt; 1">
                <xsl:value-of select="concat(' (', count, ')')"/>
              </xsl:if>
            </xsl:with-param>
            <xsl:with-param name="width" select="$namewidth + 2"/>
          </xsl:call-template>
          
          <xsl:call-template name="rpad">
            <xsl:with-param name="string" select="$source"/>
            <xsl:with-param name="width" select="$sourcewidth + 2"/>
          </xsl:call-template>
          
          <xsl:value-of select="energy"/>
          
          <xsl:value-of select="$newline"/>

        </xsl:for-each>
      
      <xsl:value-of select="$newline"/>

    </xsl:if>
      
  </xsl:template>

  <!-- Find the maximum width of a set of process names. -->
  <xsl:template name="maxprocessnamewidth">
    <xsl:param name="processes"/>
    <xsl:param name="header"/>
    <xsl:param name="width">0</xsl:param>
    
    <xsl:variable name="first" select="$processes[position() = 1]"/>
    
    <xsl:variable name="firstname">
      <xsl:value-of select="$first/name"/>
      <xsl:if test="$first/count &gt; 1">
        <xsl:value-of select="concat(' (', $first/count, ')')"/>
      </xsl:if>
    </xsl:variable>
    
    <xsl:variable name="firstwidth" select="string-length($firstname)"/>
        
    <xsl:choose>
    
      <xsl:when test="count($processes) &gt; 1">
      
        <xsl:variable name="next">
          <xsl:call-template name="maxprocessnamewidth">
            <xsl:with-param 
              name="processes" select="$processes[position() &gt; 1]"/>
            <xsl:with-param name="header" select="$header"/>
            <xsl:with-param name="width" select="$firstwidth"/>
          </xsl:call-template>
        </xsl:variable>
        
        <xsl:choose>
          <xsl:when test="$firstwidth &gt; $next">
            <xsl:value-of select="$firstwidth"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="$next"/>
          </xsl:otherwise>
        </xsl:choose>
        
      </xsl:when>
      
      <xsl:otherwise>
      
        <xsl:variable name="headerwidth" select="string-length($header)"/>
        
        <xsl:choose>
          <xsl:when test="$firstwidth &gt; $headerwidth">
            <xsl:value-of select="$firstwidth"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="$headerwidth"/>
          </xsl:otherwise>
        </xsl:choose>
      
      </xsl:otherwise>
      
    </xsl:choose>
          
  </xsl:template>

  <!-- Find the maximum width of a set of process soruces. -->
  <xsl:template name="maxprocesssourcewidth">
    <xsl:param name="processes"/>
    <xsl:param name="header"/>
    <xsl:param name="width">0</xsl:param>
    
    <xsl:variable name="first" select="$processes[position() = 1]"/>
    
    <xsl:variable name="firstpath" select="$first/path"/>
    <xsl:variable 
      name="firstsource" 
      select="/etrecheck/header/context/process[path = $firstpath]/source"/>

    <xsl:variable name="firstwidth" select="string-length($firstsource)"/>
    
    <xsl:choose>
    
      <xsl:when test="count($processes) &gt; 1">
      
        <xsl:variable name="next">
          <xsl:call-template name="maxprocesssourcewidth">
            <xsl:with-param 
              name="processes" select="$processes[position() &gt; 1]"/>
            <xsl:with-param name="header" select="$header"/>
            <xsl:with-param name="width" select="$firstwidth"/>
          </xsl:call-template>
        </xsl:variable>
        
        <xsl:choose>
          <xsl:when test="$firstwidth &gt; $next">
            <xsl:value-of select="$firstwidth"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="$next"/>
          </xsl:otherwise>
        </xsl:choose>
        
      </xsl:when>
      
      <xsl:otherwise>
      
        <xsl:variable name="headerwidth" select="string-length($header)"/>
        
        <xsl:choose>
          <xsl:when test="$firstwidth &gt; $headerwidth">
            <xsl:value-of select="$firstwidth"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="$headerwidth"/>
          </xsl:otherwise>
        </xsl:choose>
      
      </xsl:otherwise>
      
    </xsl:choose>
          
  </xsl:template>

  <!-- Print a virtual memory usage information. -->
  <xsl:template match="vm">
  
    <xsl:call-template name="header"/>

    <xsl:variable name="width">
      <xsl:call-template name="maxwidth">
        <xsl:with-param 
          name="nodes" 
          select="$strings/available_ram | $strings/free_ram | $strings/used_ram | $strings/cached_files | $strings/swap_used"/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:variable name="availableram">
      <xsl:apply-templates select="availableram" mode="units"/>
    </xsl:variable>

    <xsl:variable name="freeram">
      <xsl:apply-templates select="freeram" mode="units"/>
    </xsl:variable>

    <xsl:variable name="usedram">
      <xsl:apply-templates select="usedram" mode="units"/>
    </xsl:variable>

    <xsl:variable name="filecache">
      <xsl:apply-templates select="filecache" mode="units"/>
    </xsl:variable>

    <xsl:variable name="swapused">
      <xsl:apply-templates select="swapused" mode="units"/>
    </xsl:variable>

    <xsl:variable name="valuewidth">
      <xsl:call-template name="maxwidthunits">
        <xsl:with-param 
          name="nodes" 
          select="availableram | freeram | usedram | filecache | swapused"/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    
    <xsl:call-template name="rpad">
      <xsl:with-param name="string" select="$strings/available_ram"/>
      <xsl:with-param name="width" select="$width + 2"/>
    </xsl:call-template>

    <xsl:call-template name="lpad">
      <xsl:with-param name="string" select="$availableram"/>
      <xsl:with-param name="width" select="$valuewidth + 2"/>
    </xsl:call-template>
    <xsl:value-of select="$newline"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    <xsl:call-template name="rpad">
      <xsl:with-param name="string" select="$strings/free_ram"/>
      <xsl:with-param name="width" select="$width + 2"/>
    </xsl:call-template>

    <xsl:call-template name="lpad">
      <xsl:with-param name="string" select="$freeram"/>
      <xsl:with-param name="width" select="$valuewidth + 2"/>
    </xsl:call-template>
    <xsl:value-of select="$newline"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    <xsl:call-template name="rpad">
      <xsl:with-param name="string" select="$strings/used_ram"/>
      <xsl:with-param name="width" select="$width + 2"/>
    </xsl:call-template>

    <xsl:call-template name="lpad">
      <xsl:with-param name="string" select="$usedram"/>
      <xsl:with-param name="width" select="$valuewidth + 2"/>
    </xsl:call-template>
    <xsl:value-of select="$newline"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    <xsl:call-template name="rpad">
      <xsl:with-param name="string" select="$strings/cached_files"/>
      <xsl:with-param name="width" select="$width + 2"/>
    </xsl:call-template>

    <xsl:call-template name="lpad">
      <xsl:with-param name="string" select="$filecache"/>
      <xsl:with-param name="width" select="$valuewidth + 2"/>
    </xsl:call-template>
    <xsl:value-of select="$newline"/>
    
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    <xsl:call-template name="rpad">
      <xsl:with-param name="string" select="$strings/swap_used"/>
      <xsl:with-param name="width" select="$width + 2"/>
    </xsl:call-template>

    <xsl:call-template name="lpad">
      <xsl:with-param name="string" select="$swapused"/>
      <xsl:with-param name="width" select="$valuewidth + 2"/>
    </xsl:call-template>

    <xsl:value-of select="$newline"/>
    <xsl:value-of select="$newline"/>
      
  </xsl:template>

  <!-- Print software install information. -->
  <xsl:template match="install">
  
    <xsl:if test="count(package) &gt; 0">
      <xsl:call-template name="header"/>

      <xsl:variable name= "namewidth">
        <xsl:call-template name="maxwidth">
          <xsl:with-param 
            name="nodes" select="$strings/install/name | package/name"/>
        </xsl:call-template>
      </xsl:variable>
            
      <xsl:variable name= "versionwidth">
        <xsl:call-template name="maxwidth">
          <xsl:with-param 
            name="nodes" 
            select="$strings/install/version | package/version"/>
        </xsl:call-template>
      </xsl:variable>

      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="2"/>
      </xsl:call-template>

      <xsl:call-template name="rpad">
        <xsl:with-param name="string" select="$strings/install/name"/>
        <xsl:with-param name="width" select="$namewidth + 2"/>
      </xsl:call-template>
        
      <xsl:call-template name="rpad">
        <xsl:with-param name="string" select="$strings/install/version"/>
        <xsl:with-param name="width" select="$versionwidth + 2"/>
      </xsl:call-template>
            
      <xsl:value-of select="$strings/install/date"/>
      <xsl:value-of select="$newline"/>
        
      <xsl:for-each select="package">

        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>

        <xsl:call-template name="rpad">
          <xsl:with-param name="string" select="name"/>
          <xsl:with-param name="width" select="$namewidth + 2"/>
        </xsl:call-template>
          
        <xsl:call-template name="rpad">
          <xsl:with-param name="string" select="version"/>
          <xsl:with-param name="width" select="$versionwidth + 2"/>
        </xsl:call-template>
        
        <xsl:apply-templates select="installdate"/>
        <xsl:value-of select="$newline"/>

      </xsl:for-each>
      
      <xsl:value-of select="$newline"/>
      
    </xsl:if>
   
  </xsl:template>

  <!-- Print diagnostics information. -->
  <xsl:template match="diagnostics">
  
    <xsl:call-template name="header"/>
    
    <xsl:apply-templates select="event"/>
    
    <xsl:if test="insufficientpermissions = 'true'">
      
      <xsl:call-template name="insufficientpermissions"/>
      
    </xsl:if>
    
  </xsl:template>

  <!-- Print insufficient permissions. -->
  <xsl:template name="insufficientpermissions">
  
    <xsl:value-of select="$newline"/>
    
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    <xsl:copy-of select="$strings/diagnostics/insufficientpermissions"/>
    <xsl:value-of select="$newline"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    <xsl:copy-of select="$strings/diagnostics/runasadmin"/>
    <xsl:value-of select="$newline"/>

  </xsl:template>
  
  <!-- Print a diagnostics event. -->
  <xsl:template match="event">
  
    <xsl:variable name="type" select="type"/>
    
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    <xsl:value-of select="date"/>
    <xsl:text> </xsl:text>
    
    <xsl:choose>
      
      <xsl:when test="type = 10">
        <xsl:value-of 
          select="$strings/diagnostics/types/value[@key = $type]"/>
        <xsl:text> </xsl:text>
        <xsl:apply-templates select="name"/>
      </xsl:when>
      
      <xsl:otherwise>
        <xsl:apply-templates select="name"/>
        <xsl:text> </xsl:text>
        <xsl:value-of 
          select="$strings/diagnostics/types/value[@key = $type]"/>
      </xsl:otherwise>
    
    </xsl:choose>
      
    <xsl:value-of select="$newline"/>
    
    <xsl:if test="information">

      <!-- ASC scrambles multi-line elements. Do the scrambling myself 
           so I have some control over the outcome. -->
      <xsl:call-template name="split">
        <xsl:with-param name="text">
          <xsl:value-of select="information"/>
        </xsl:with-param>
        <xsl:with-param name="indent" select="4"/>
      </xsl:call-template>
      
    </xsl:if>
    
  </xsl:template>
  
  <!-- Print a EtreCheck deleted files. -->
  <xsl:template match="etrecheckdeletedfiles">
  
    <xsl:if test="count(deletedfile) &gt; 0">
      <xsl:call-template name="header"/>

      <table>
        <xsl:call-template name="tablestyle"/>
        
        <xsl:for-each select="deletedfile">
          <tr>
            <td>
              <xsl:call-template name="cellstyle">          
                <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
              </xsl:call-template>

              <xsl:value-of select="date"/>
            </td>
            <td>
              <xsl:call-template name="cellstyle">          
                <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
              </xsl:call-template>

              <xsl:value-of select="path_safe"/>
            </td>
            <td>
              <xsl:call-template name="cellstyle">          
                <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
              </xsl:call-template>

              <xsl:value-of select="reason"/>
            </td>
          </tr>
        </xsl:for-each>
        
      </table>
      
    </xsl:if>
      
  </xsl:template>

  <!-- Print a device node. -->
  <xsl:template match="node">

    <xsl:if test="manufacturer">
      <xsl:value-of select="manufacturer"/>
      <xsl:text> </xsl:text>
    </xsl:if>
    
    <xsl:if test="name">
      <xsl:value-of select="name"/>
    </xsl:if>
    <xsl:value-of select="$newline"/>

    <xsl:apply-templates select="node"/>
    
  </xsl:template>
  
  <!-- Create an HTML CSS style attribute suitable for a text report. -->
  <xsl:template name="style">
    <xsl:param name="indent">0</xsl:param>
    <xsl:param name="color"/>
    <xsl:param name="style"/>
    <xsl:param name="css"/>
  
    <xsl:attribute name="style">
    
      <xsl:text>text-indent: </xsl:text>
      <xsl:call-template name="indent">
        <xsl:with-param name="indent">
          <xsl:value-of select="$indent"/>
        </xsl:with-param>
      </xsl:call-template>
      <xsl:text>px;</xsl:text>
      
      <xsl:call-template name="base_style">
        <xsl:with-param name="color" select="$color"/>
        <xsl:with-param name="color" select="$style"/>
        <xsl:with-param name="color" select="$css"/>
      </xsl:call-template>

    </xsl:attribute>

  </xsl:template>
  
  <!-- Create an HTML CSS style attribute suitable for a table cell in the 
       text report. -->
  <xsl:template name="cellstyle">
    <xsl:param name="color"/>
    <xsl:param name="style"/>
    <xsl:param name="css"/>
  
  </xsl:template>

  <!-- Print a number with units right aligned. -->
  <xsl:template name="unitsstyle">
    <xsl:param name="css"/>
  
    <xsl:attribute name="style">

      <xsl:call-template name="base_style">
        <xsl:with-param name="css">
          <xsl:value-of select="concat($css,'text-align: right; ')"/>
        </xsl:with-param>
      </xsl:call-template>

    </xsl:attribute>

  </xsl:template>

  <!-- Create an HTML CSS style attribute suitable for a text report. -->
  <xsl:template name="base_style">
    <xsl:param name="indent">0</xsl:param>
    <xsl:param name="color"/>
    <xsl:param name="style"/>
    <xsl:param name="css"/>
        
  </xsl:template>

  <!-- Generate an indent. -->
  <xsl:template name="indent">
    <xsl:param name="indent">0</xsl:param>
      
    <xsl:value-of select="substring($pad, 1, $indent)"/>
    
  </xsl:template>
  
  <!-- Right pad a string. -->
  <xsl:template name="rpad">
    <xsl:param name="string"/>
    <xsl:param name="width"/>
      
    <xsl:variable name="stringwidth" select="string-length($string)"/>
    
    <xsl:variable name="padwidth">
      <xsl:choose>
        <xsl:when test="$stringwidth &gt; $width">
          <xsl:value-of select="$stringwidth"/>
        </xsl:when>
        
        <xsl:otherwise>
          <xsl:value-of select="$width"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    
    <xsl:value-of select="substring(concat($string, $pad), 1, $padwidth)"/>
    
  </xsl:template>
  
  <!-- Left pad a string. -->
  <xsl:template name="lpad">
    <xsl:param name="string"/>
    <xsl:param name="width"/>
      
    <xsl:variable name="stringwidth" select="string-length($string)"/>
    
    <xsl:variable name="padwidth">
      <xsl:choose>
        <xsl:when test="$stringwidth &gt; $width">
          <xsl:value-of select="$stringwidth"/>
        </xsl:when>
        
        <xsl:otherwise>
          <xsl:value-of select="$width - $stringwidth"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:value-of select="concat(substring($pad, 1, $padwidth), $string)"/>
    
  </xsl:template>
  
  <!-- Center a string in a field. -->
  <xsl:template name="center">
    <xsl:param name="string"/>
    <xsl:param name="width"/>
      
    <xsl:variable name="stringwidth" select="string-length($string)"/>
    
    <xsl:variable name="padwidth">
      <xsl:choose>
        <xsl:when test="$stringwidth &gt; $width">
          <xsl:value-of select="$stringwidth"/>
        </xsl:when>
        
        <xsl:otherwise>
          <xsl:value-of select="($width - $stringwidth) div 2"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:value-of select="concat(substring($pad, 1, $padwidth), $string)"/>
    
  </xsl:template>

  <!-- Find the maximum width of a nodeset. -->
  <xsl:template name="maxwidth">
    <xsl:param name="nodes"/>
    <xsl:param name="width">0</xsl:param>
    
    <xsl:variable name="strings" select="exslt:node-set($nodes)"/>

    <xsl:variable 
      name="firstwidth" select="string-length($strings[position() = 1])"/>
        
    <xsl:choose>
    
      <xsl:when test="count($strings) &gt; 1">
      
        <xsl:variable name="next">
          <xsl:call-template name="maxwidth">
            <xsl:with-param 
              name="nodes" select="$strings[position() &gt; 1]"/>
            <xsl:with-param name="width" select="$firstwidth"/>
          </xsl:call-template>
        </xsl:variable>
        
        <xsl:choose>
          <xsl:when test="$firstwidth &gt; $next">
            <xsl:value-of select="$firstwidth"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="$next"/>
          </xsl:otherwise>
        </xsl:choose>
        
      </xsl:when>
      
      <xsl:otherwise>
      
        <xsl:value-of select="$firstwidth"/>
      
      </xsl:otherwise>
      
    </xsl:choose>
          
  </xsl:template>
  
  <!-- Find the maximum width of a nodeset with units. -->
  <xsl:template name="maxwidthunits">
    <xsl:param name="nodes"/>
    <xsl:param name="width">0</xsl:param>
    
    <xsl:variable name="first" select="$nodes[position() = 1]"/>

    <xsl:variable name="firstvalue">
      <xsl:apply-templates select="$first" mode="units"/>
    </xsl:variable>
    
    <xsl:variable name="firstwidth" select="string-length($firstvalue)"/>
        
    <xsl:choose>
    
      <xsl:when test="count($strings) &gt; 1">
      
        <xsl:variable name="next">
          <xsl:call-template name="maxwidth">
            <xsl:with-param 
              name="nodes" select="$nodes[position() &gt; 1]"/>
            <xsl:with-param name="width" select="$firstwidth"/>
          </xsl:call-template>
        </xsl:variable>
        
        <xsl:choose>
          <xsl:when test="$firstwidth &gt; $next">
            <xsl:value-of select="$firstwidth"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="$next"/>
          </xsl:otherwise>
        </xsl:choose>
        
      </xsl:when>
      
      <xsl:otherwise>
      
        <xsl:value-of select="$firstwidth"/>
      
      </xsl:otherwise>
      
    </xsl:choose>
          
  </xsl:template>

  <!-- Apply a table style to match other output. -->
  <xsl:template name="tablestyle">
    <xsl:param name="margin-left">20px</xsl:param>
    
  </xsl:template>

  <!-- ASC scrambles embedded new lines. Can't fix it with CSS. -->
  <xsl:template name="split">
    <xsl:param name="text"/>
    <xsl:param name="indent">0</xsl:param>

    <xsl:if test="string-length($text) &gt; 0">
      
      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="$indent"/>
      </xsl:call-template>
      
      <xsl:choose>
      
        <xsl:when test="contains($text, $newline)">
          <xsl:variable name="before">
            <xsl:value-of select="substring-before($text, $newline)"/>
          </xsl:variable>

          <xsl:variable name="after">
            <xsl:value-of select="substring-after($text, $newline)"/>
          </xsl:variable>

          <xsl:if test="normalize-space($before) != ''">
            <xsl:value-of select="$before"/>
            <xsl:value-of select="$newline"/>
          </xsl:if>

          <xsl:call-template name="split">
            <xsl:with-param name="text" select="$after"/>
          </xsl:call-template>
        </xsl:when>
        
        <xsl:otherwise>
          <xsl:value-of select="$text"/>
          <xsl:value-of select="$newline"/>
        </xsl:otherwise>
      
      </xsl:choose>
      
    </xsl:if>
  </xsl:template>

  <!-- Print a section header. -->
  <xsl:template name="header">
    <xsl:param name="key"><xsl:value-of select="local-name(.)"/></xsl:param>
    
    <xsl:call-template name="custom_header">
      <xsl:with-param 
        name="text" select="$strings/headers/value[@key = $key]"/>
    </xsl:call-template>
    
  </xsl:template>
  
  <!-- Print a custom section header. -->
  <xsl:template name="custom_header">
    <xsl:param name="text"/>
    
    <xsl:value-of select="$text"/>
    <xsl:value-of select="$newline"/>
    
  </xsl:template>

  <!-- Print only the day portion of an install date. -->
  <xsl:template match="installdate">
  
    <xsl:value-of select="substring(., 1, 10)"/>
    
  </xsl:template>
      
  <!-- Print a label. -->
  <xsl:template match="*" mode="label">
  
    <strong>
      <xsl:copy-of select="."/>
    </strong>
    
  </xsl:template>
    
  <xsl:include href="Library.xsl" />

</xsl:stylesheet>
